@@ -22,7 +22,7 @@ import posixpath
 import textwrap
 import importlib
 
-VERSION = '4.1.1'
+VERSION = '4.2'
 ENCODING = 'utf-8'
 PY3 = sys.version_info >= (3,)
 PACKAGE_DIR = os.path.dirname(__file__)
@@ -72,7 +72,7 @@ def get_chr(code):
 
 def get_class(name_spec):
     if ':' in name_spec:
-        path, name = name_spec.split(':')
+        path, name = name_spec.rsplit(':', 1)
         if path not in sys.path:
             sys.path.insert(0, os.path.expanduser(path))
     else:
