@@ -17,7 +17,10 @@
 # SkoolKit. If not, see <http://www.gnu.org/licenses/>.
 
 from . import warn, get_int_param, open_file
-from .skoolctl import AD_START, AD_WRITER, AD_ORG, AD_END, AD_SET
+from .skoolctl import AD_START, AD_WRITER, AD_ORG, AD_END, AD_SET, AD_IGNOREUA
+from .skoolctl import TITLE, DESCRIPTION, REGISTERS, MID_BLOCK, INSTRUCTION, END
+
+COMMENT_TYPES = (TITLE, DESCRIPTION, REGISTERS, MID_BLOCK, INSTRUCTION, END)
 
 class CtlParserError(Exception):
     pass
@@ -73,6 +76,8 @@ class CtlParser:
         self.multiline_comments = {}
         self.entry_asm_directives = {}
         self.instruction_asm_directives = {}
+        self.ignoreua_directives = {}
+        self.loops = []
 
     def parse_ctl(self, ctlfile):
         entry_ctl = None
@@ -98,19 +103,31 @@ class CtlParser:
                 elif ctl == 'E':
                     self.end_comments.setdefault(start, []).append(text)
                 elif ctl == 'R':
-                    fields = text.split(' ', 1)
-                    valid = len(fields) == 2
-                    if valid:
-                        self.registers.setdefault(start, [])
-                        self.registers[start].append(fields)
+                    if text:
+                        fields = text.split(' ', 1)
+                        if len(fields) == 1:
+                            fields.append('')
+                        self.registers.setdefault(start, []).append(fields)
                 elif ctl == 'M':
                     self.multiline_comments[start] = (end, text)
+                elif ctl == 'L':
+                    count = lengths[0][0]
+                    if count > 1:
+                        if len(lengths) > 1:
+                            repeat_entries = lengths[1][0]
+                        else:
+                            repeat_entries = 0
+                        loop_end = start + count * (end - start + 1)
+                        if loop_end > 65536:
+                            warn('Loop crosses 64K boundary:\n{}'.format(s_line))
+                        self.loops.append((start, end, count, repeat_entries))
+                        self.subctls[loop_end] = None
                 else:
                     self.subctls[start] = ctl.lower()
                     self.instruction_comments[start] = text
                     if end:
                         self.subctls[end + 1] = None
-                if lengths:
+                if ctl != 'L' and lengths:
                     self.lengths[start] = lengths
             elif asm_directive:
                 directive, address, value = asm_directive
@@ -119,6 +136,7 @@ class CtlParser:
                 else:
                     self.instruction_asm_directives.setdefault(address, []).append((directive, value))
         f.close()
+        self._unroll_loops()
 
     def _parse_ctl_line(self, line, entry_ctl):
         ctl = start = end = text = asm_directive = None
@@ -126,7 +144,7 @@ class CtlParser:
         first_char = line[0]
         content = line[1:].lstrip()
         if content:
-            if first_char in ' bBcCDEgiMRsStTuwW':
+            if first_char in ' bBcCDEgiLMRsStTuwW':
                 fields = content.split(' ', 1)
                 ctl = first_char
                 if ctl == ' ':
@@ -157,27 +175,72 @@ class CtlParser:
                 else:
                     end = None
                 lengths = int_params[2:]
+                if ctl == 'L':
+                    if end is None:
+                        raise CtlParserError("loop length not specified")
+                    if not lengths:
+                        raise CtlParserError("loop count not specified")
                 if len(fields) > 1:
                     text = fields[1]
             elif first_char == ';' and content.startswith('@'):
                 asm_fields = content.split(':', 1)
                 if len(asm_fields) < 2:
                     raise CtlParserError("invalid ASM directive declaration")
+                directive = asm_fields[0][1:]
                 asm_fields[1] = asm_fields[1].split('=', 1)
+                addr_str = asm_fields[1][0]
+                if directive == AD_IGNOREUA:
+                    comment_type = 'i'
+                    if ':' in addr_str:
+                        addr_str, comment_type = addr_str.split(':', 1)
                 try:
-                    address = get_int_param(asm_fields[1][0])
+                    address = get_int_param(addr_str)
                 except ValueError:
                     raise CtlParserError("invalid ASM directive address")
-                directive = asm_fields[0][1:]
                 if len(asm_fields[1]) == 2:
                     value = asm_fields[1][1]
                 else:
                     value = None
-                asm_directive = (directive, address, value)
+                if directive == AD_IGNOREUA:
+                    if comment_type not in COMMENT_TYPES:
+                        raise CtlParserError("invalid @ignoreua directive address suffix: '{}:{}'".format(addr_str, comment_type))
+                    self.ignoreua_directives.setdefault(address, set()).add(comment_type)
+                else:
+                    asm_directive = (directive, address, value)
             elif first_char not in '#%;':
                 raise CtlParserError("invalid directive")
         return ctl, start, end, text, lengths, asm_directive
 
+    def _unroll_loops(self):
+        for start, end, count, repeat_entries in self.loops:
+            for directives in (self.subctls, self.instruction_comments, self.lengths):
+                self._repeat_directives(directives, start, end, count)
+            self._repeat_multiline_comments(start, end, count)
+            mid_block_comments = {k: v for k, v in self.comments.items() if start <= k <= end and k not in self.ctls}
+            self._repeat_directives(mid_block_comments, start, end, count)
+            self.comments.update(mid_block_comments)
+            if repeat_entries:
+                for directives in (self.ctls, self.titles, self.registers, self.end_comments):
+                    self._repeat_directives(directives, start, end, count)
+                entry_descriptions = {k: v for k, v in self.comments.items() if start <= k <= end and k in self.ctls}
+                self._repeat_directives(entry_descriptions, start, end, count)
+                self.comments.update(entry_descriptions)
+
+    def _repeat_directives(self, directives, start, end, count):
+        interval = end - start + 1
+        repeated = {k: v for k, v in directives.items() if start <= k <= end}
+        for addr, value in repeated.items():
+            for i in range(1, count):
+                directives[addr + i * interval] = value
+
+    def _repeat_multiline_comments(self, start, end, count):
+        interval = end - start + 1
+        repeated = {k: v for k, v in self.multiline_comments.items() if start <= k <= end}
+        for addr, (mlc_end, comment) in repeated.items():
+            for i in range(1, count):
+                offset = i * interval
+                self.multiline_comments[addr + offset] = (mlc_end + offset, comment)
+
     def get_block_title(self, address):
         return self.titles.get(address)
 
@@ -194,7 +257,8 @@ class CtlParser:
         return self.end_comments.get(address, ())
 
     def get_lengths(self, address):
-        return self.lengths.get(address)
+        if address in self.lengths:
+            return self.lengths[address][:]
 
     def get_multiline_comment(self, address):
         return self.multiline_comments.get(address, (None, None))
@@ -211,10 +275,13 @@ class CtlParser:
                 if directive == asm_dir:
                     return True
 
+    def has_ignoreua_directive(self, address, comment_type):
+        return comment_type in self.ignoreua_directives.get(address, ())
+
     def get_blocks(self):
         # Create top-level blocks
         blocks = []
-        for address in sorted(self.ctls.keys()):
+        for address in sorted([k for k in self.ctls if k < 65536]):
             blocks.append(Block(self.ctls[address], address, True))
 
         # Set top-level block end addresses
