@@ -19,7 +19,7 @@
 import re
 import inspect
 
-from . import warn, write_text, wrap, get_chr, SkoolParsingError
+from . import warn, write_text, wrap, get_chr, SkoolKitError, SkoolParsingError
 from . import skoolmacro
 from .skoolmacro import MacroParsingError, UnsupportedMacroError
 from .skoolparser import TableParser, ListParser, TABLE_MARKER, TABLE_END_MARKER, LIST_MARKER, LIST_END_MARKER
@@ -124,24 +124,14 @@ class AsmWriter:
             self.write_line('')
 
     def print_entry(self):
-        self.print_comment_lines([self.entry.description], ignoreua=self.entry.ignoretua)
+        self.print_comment_lines([self.entry.description], ignoreua=self.entry.ignoreua['t'])
         if self.entry.details:
-            self.print_comment_lines(self.entry.details, ignoreua=self.entry.ignoredua, started=True)
+            self.print_comment_lines(self.entry.details, ignoreua=self.entry.ignoreua['d'], started=True)
         if self.entry.registers:
-            self.write_line(';')
-            prefix_len = max([len(reg.prefix) for reg in self.entry.registers])
-            if prefix_len:
-                prefix_len += 1
-            indent = ''.ljust(prefix_len)
-            for reg in self.entry.registers:
-                if reg.prefix:
-                    prefix = '{0}:'.format(reg.prefix).rjust(prefix_len)
-                else:
-                    prefix = indent
-                self.write_line('; {0}{1} {2}'.format(prefix, reg.name, self.expand(reg.contents)))
+            self.print_registers()
         self.print_instructions()
         if self.entry.end_comment:
-            self.print_comment_lines(self.entry.end_comment, ignoreua=self.entry.ignoreecua)
+            self.print_comment_lines(self.entry.end_comment, ignoreua=self.entry.ignoreua['e'])
 
     def write_line(self, s):
         write_text('{0}{1}'.format(s, self.end))
@@ -150,6 +140,8 @@ class AsmWriter:
         """Discard the current memory snapshot and replace it with the one that
         was most recently saved (by
         :meth:`~skoolkit.skoolasm.AsmWriter.push_snapshot`)."""
+        if len(self._snapshots) < 2:
+            raise SkoolKitError("Cannot pop snapshot when snapshot stack is empty")
         self.snapshot = self._snapshots.pop()[0]
 
     def push_snapshot(self, name=''):
@@ -375,6 +367,34 @@ class AsmWriter:
                             self.warn('Comment contains address ({0}) not converted to a label:\n; {1}'.format(uaddress, line))
                 self.write_line('; {0}'.format(line).rstrip())
 
+    def print_registers(self):
+        self.write_line(';')
+        prefix_len = max([len(reg.prefix) for reg in self.entry.registers])
+        if prefix_len:
+            prefix_len += 1
+        indent = ''.ljust(prefix_len)
+        for reg in self.entry.registers:
+            if reg.prefix:
+                prefix = '{}:'.format(reg.prefix).rjust(prefix_len)
+            else:
+                prefix = indent
+            reg_label = prefix + reg.name
+            reg_desc = self.expand(reg.contents)
+            if not self.entry.ignoreua['r']:
+                uaddress = self.find_unconverted_address(reg_desc)
+                if uaddress:
+                    line = '; {} {}'.format(reg_label, reg_desc)
+                    self.warn('Register description contains address ({}) not converted to a label:\n{}'.format(uaddress, line))
+            if reg_desc:
+                desc_indent = len(reg_label) + 1
+                desc_lines = wrap(reg_desc, self.desc_width - desc_indent)
+                self.write_line('; {} {}'.format(reg_label, desc_lines[0]))
+                desc_prefix = ''.ljust(desc_indent)
+                for line in desc_lines[1:]:
+                    self.write_line('; {}{}'.format(desc_prefix, line))
+            else:
+                self.write_line('; {}'.format(reg_label))
+
     def print_instruction_prefix(self, instruction):
         mid_routine_comment = instruction.get_mid_routine_comment()
         if mid_routine_comment:
