@@ -65,21 +65,21 @@ DEF_MEMORY_MAP_ENTRY_TYPES = 'bcgstuw'
 
 # Byte flip table
 FLIP = (
-    0, 128, 64, 192, 32, 160, 96, 224, 16, 144, 80, 208, 48, 176, 112, 240, 
-    8, 136, 72, 200, 40, 168, 104, 232, 24, 152, 88, 216, 56, 184, 120, 248, 
-    4, 132, 68, 196, 36, 164, 100, 228, 20, 148, 84, 212, 52, 180, 116, 244, 
-    12, 140, 76, 204, 44, 172, 108, 236, 28, 156, 92, 220, 60, 188, 124, 252, 
-    2, 130, 66, 194, 34, 162, 98, 226, 18, 146, 82, 210, 50, 178, 114, 242, 
-    10, 138, 74, 202, 42, 170, 106, 234, 26, 154, 90, 218, 58, 186, 122, 250, 
-    6, 134, 70, 198, 38, 166, 102, 230, 22, 150, 86, 214, 54, 182, 118, 246, 
-    14, 142, 78, 206, 46, 174, 110, 238, 30, 158, 94, 222, 62, 190, 126, 254, 
-    1, 129, 65, 193, 33, 161, 97, 225, 17, 145, 81, 209, 49, 177, 113, 241, 
-    9, 137, 73, 201, 41, 169, 105, 233, 25, 153, 89, 217, 57, 185, 121, 249, 
-    5, 133, 69, 197, 37, 165, 101, 229, 21, 149, 85, 213, 53, 181, 117, 245, 
-    13, 141, 77, 205, 45, 173, 109, 237, 29, 157, 93, 221, 61, 189, 125, 253, 
-    3, 131, 67, 195, 35, 163, 99, 227, 19, 147, 83, 211, 51, 179, 115, 243, 
-    11, 139, 75, 203, 43, 171, 107, 235, 27, 155, 91, 219, 59, 187, 123, 251, 
-    7, 135, 71, 199, 39, 167, 103, 231, 23, 151, 87, 215, 55, 183, 119, 247, 
+    0, 128, 64, 192, 32, 160, 96, 224, 16, 144, 80, 208, 48, 176, 112, 240,
+    8, 136, 72, 200, 40, 168, 104, 232, 24, 152, 88, 216, 56, 184, 120, 248,
+    4, 132, 68, 196, 36, 164, 100, 228, 20, 148, 84, 212, 52, 180, 116, 244,
+    12, 140, 76, 204, 44, 172, 108, 236, 28, 156, 92, 220, 60, 188, 124, 252,
+    2, 130, 66, 194, 34, 162, 98, 226, 18, 146, 82, 210, 50, 178, 114, 242,
+    10, 138, 74, 202, 42, 170, 106, 234, 26, 154, 90, 218, 58, 186, 122, 250,
+    6, 134, 70, 198, 38, 166, 102, 230, 22, 150, 86, 214, 54, 182, 118, 246,
+    14, 142, 78, 206, 46, 174, 110, 238, 30, 158, 94, 222, 62, 190, 126, 254,
+    1, 129, 65, 193, 33, 161, 97, 225, 17, 145, 81, 209, 49, 177, 113, 241,
+    9, 137, 73, 201, 41, 169, 105, 233, 25, 153, 89, 217, 57, 185, 121, 249,
+    5, 133, 69, 197, 37, 165, 101, 229, 21, 149, 85, 213, 53, 181, 117, 245,
+    13, 141, 77, 205, 45, 173, 109, 237, 29, 157, 93, 221, 61, 189, 125, 253,
+    3, 131, 67, 195, 35, 163, 99, 227, 19, 147, 83, 211, 51, 179, 115, 243,
+    11, 139, 75, 203, 43, 171, 107, 235, 27, 155, 91, 219, 59, 187, 123, 251,
+    7, 135, 71, 199, 39, 167, 103, 231, 23, 151, 87, 215, 55, 183, 119, 247,
     15, 143, 79, 207, 47, 175, 111, 239, 31, 159, 95, 223, 63, 191, 127, 255
 )
 
@@ -394,6 +394,8 @@ class HtmlWriter:
         """Discard the current memory snapshot and replace it with the one that
         was most recently saved (by
         :meth:`~skoolkit.skoolhtml.HtmlWriter.push_snapshot`)."""
+        if len(self._snapshots) < 2:
+            raise SkoolKitError("Cannot pop snapshot when snapshot stack is empty")
         self.snapshot = self._snapshots.pop()[0]
 
     def push_snapshot(self, name=''):
@@ -651,7 +653,7 @@ class HtmlWriter:
         for item, subitems in items:
             if subitems:
                 item = '{}\n{}\n'.format(item, self._build_changelog_items(subitems, level + 1))
-            changelog_items.append(self.format_template('changelog_item', {'item': item}))
+            changelog_items.append(self.format_template('changelog_item', {'item': item}, 'list_item'))
         if level > 0:
             indent = level
         else:
@@ -692,7 +694,7 @@ class HtmlWriter:
                 't_anchor': self.format_anchor(title),
                 'num': 1 + j % 2,
                 'title': title,
-                'description': description,
+                'description': self.expand(description, cwd),
                 't_changelog_item_list': self._build_changelog_items(changelog_items)
             }
             entries.append(self.format_template('changelog_entry', t_changelog_entry_subs))
@@ -986,53 +988,31 @@ class HtmlWriter:
                 self.macros[macro] = method
 
     def build_table(self, table):
-        table_class = table.table_class
-        html = []
-        if table_class:
-            class_attr = ' class="{0}"'.format(table_class)
-        else:
-            class_attr = ''
-        html.append('<table{0}>'.format(class_attr))
+        rows = []
         for row in table.rows:
-            html.append('<tr>')
+            cells = []
             for cell in row:
-                if cell.colspan > 1:
-                    colspan = ' colspan="{0}"'.format(cell.colspan)
-                else:
-                    colspan = ''
-                if cell.rowspan > 1:
-                    rowspan = ' rowspan="{0}"'.format(cell.rowspan)
-                else:
-                    rowspan = ''
+                cell_subs = {
+                    'colspan': cell.colspan,
+                    'rowspan': cell.rowspan,
+                    'contents': cell.contents
+                }
                 if cell.header:
-                    tag = 'th'
-                    td_class = ''
+                    cells.append(self.format_template('table_header_cell', cell_subs))
                 else:
-                    tag = 'td'
                     cell_class = cell.cell_class
                     if cell.transparent:
                         cell_class += " transparent"
-                    cell_class = cell_class.lstrip()
-                    if cell_class:
-                        td_class = ' class="{0}"'.format(cell_class)
-                    else:
-                        td_class = ''
-                html.append('<{0}{1}{2}{3}>{4}</{0}>'.format(tag, td_class, colspan, rowspan, cell.contents))
-            html.append('</tr>')
-        html.append('</table>')
-        return '\n'.join(html)
+                    cell_subs['class'] = cell_class.lstrip()
+                    cells.append(self.format_template('table_cell', cell_subs))
+            rows.append(self.format_template('table_row', {'cells': '\n'.join(cells)}))
+        table_subs = {'class': table.table_class, 'm_table_row': '\n'.join(rows)}
+        return self.format_template('table', table_subs)
 
     def build_list(self, list_obj):
-        html = []
-        if list_obj.css_class:
-            class_attr = ' class="{0}"'.format(list_obj.css_class)
-        else:
-            class_attr = ''
-        html.append('<ul{0}>'.format(class_attr))
-        for item in list_obj.items:
-            html.append('<li>{0}</li>'.format(item))
-        html.append('</ul>')
-        return '\n'.join(html)
+        items = [self.format_template('list_item', {'item': i}) for i in list_obj.items]
+        list_subs = {'class': list_obj.css_class, 'm_list_item': '\n'.join(items)}
+        return self.format_template('list', list_subs)
 
     def _get_udg_addresses(self, addr_spec, width):
         if type(addr_spec) is int:
@@ -1103,8 +1083,13 @@ class HtmlWriter:
                 return join(self.paths[path_id], '{0}{1}'.format(fname, suffix))
             raise SkoolKitError("Unknown path ID '{0}' for image file '{1}'".format(path_id, fname))
 
-    def parse_image_params(self, text, index, num=0, defaults=(), path_id=DEF_IMG_PATH, fname='', chars='', ints=None, names=()):
-        """Parse a string of the form ``params[{x,y,width,height}][(fname)]``.
+    def parse_image_params(self, text, index, num=0, defaults=(),
+                           path_id=DEF_IMG_PATH, fname='', chars='', ints=None,
+                           names=(), frame=False, alt=False):
+        """Parse a string of the form:
+
+        ``params[{x,y,width,height}][(fname[*frame][|alt])]``
+
         The parameter string ``params`` may contain comma-separated values and
         is parsed until either the end is reached, or an invalid character is
         encountered. The default set of valid characters consists of the comma,
@@ -1128,13 +1113,19 @@ class HtmlWriter:
                      must evaluate to an integer.
         :param names: The names of the parameters; if not empty, keyword
                       arguments in the parameter string ``params`` are parsed.
+        :param frame: Whether to parse and return the frame name.
+        :param alt: Whether to parse and return the alt text.
         :return: A list of the form
-                 ``[end, image_path, crop_rect, value1, value2...]``, where:
+                 ``[end, image_path, frame, alt, crop_rect, value1, value2...]``,
+                 where:
 
                  * ``end`` is the index at which parsing terminated
                  * ``image_path`` is either the full path of the image file
                    (relative to the root directory of the disassembly) or
                    ``fname`` (if `path_id` is blank or `None`)
+                 * ``frame`` is the frame name (present only if `frame` is
+                   `True`)
+                 * ``alt`` is the alt text (present only if `alt` is `True`)
                  * ``crop_rect`` is ``(x, y, width, height)``
                  * ``value1``, ``value2`` etc. are the parameter values
         """
@@ -1153,14 +1144,28 @@ class HtmlWriter:
         else:
             x, y, width, height = defaults
 
+        alt_text = None
+        frame_name = None
         if p_text:
-            fname = p_text
+            if alt and '|' in p_text:
+                p_text, alt_text = p_text.split('|', 1)
+            if frame and '*' in p_text:
+                p_text, frame_name = p_text.split('*', 1)
+            if p_text:
+                fname = p_text
+            if frame_name == '':
+                frame_name = fname
         if path_id:
             img_path = self.image_path(fname, path_id)
         else:
             img_path = fname
 
-        return [end, img_path, (x, y, width, height)] + params
+        retval = [end, img_path, (x, y, width, height)] + params
+        if alt:
+            retval.insert(2, alt_text)
+        if frame:
+            retval.insert(2, frame_name)
+        return retval
 
     def _expand_item_macro(self, item, link_text, cwd, items, path_id):
         if item and link_text == '':
@@ -1207,12 +1212,12 @@ class HtmlWriter:
 
         param_names = ('addr', 'chars', 'attr', 'scale')
         defaults = (len(message), 56, 2)
-        params = self.parse_image_params(text, index, defaults=defaults, path_id='FontImagePath', fname='font', names=param_names)
-        end, img_path, crop_rect, addr, chars, attr, scale = params
+        params = self.parse_image_params(text, index, defaults=defaults, path_id='FontImagePath', fname='font', names=param_names, alt=True)
+        end, img_path, alt, crop_rect, addr, chars, attr, scale = params
         if self.need_image(img_path):
             udg_array = self.get_font_udg_array(addr, attr, message[:chars])
             self.write_image(img_path, udg_array, crop_rect, scale)
-        return end, self.img_element(cwd, img_path)
+        return end, self.img_element(cwd, img_path, alt)
 
     def expand_html(self, text, index, cwd):
         return skoolmacro.parse_html(text, index)
@@ -1277,12 +1282,12 @@ class HtmlWriter:
         # #SCR[scale,x,y,w,h,df,af][{x,y,width,height}][(fname)]
         param_names = ('scale', 'x', 'y', 'w', 'h', 'df', 'af')
         defaults = (1, 0, 0, 32, 24, 16384, 22528)
-        params = self.parse_image_params(text, index, defaults=defaults, path_id='ScreenshotImagePath', fname='scr', names=param_names)
-        end, scr_path, crop_rect, scale, x, y, w, h, df, af = params
+        params = self.parse_image_params(text, index, defaults=defaults, path_id='ScreenshotImagePath', fname='scr', names=param_names, alt=True)
+        end, scr_path, alt, crop_rect, scale, x, y, w, h, df, af = params
         if self.need_image(scr_path):
             udgs = self.screenshot(x, y, w, h, df, af)
             self.write_image(scr_path, udgs, crop_rect, scale)
-        return end, self.img_element(cwd, scr_path)
+        return end, self.img_element(cwd, scr_path, alt)
 
     def expand_space(self, text, index, cwd):
         end, num_sp = skoolmacro.parse_space(text, index)
@@ -1298,11 +1303,11 @@ class HtmlWriter:
         path_id = 'UDGImagePath'
         param_names = ('addr', 'attr', 'scale', 'step', 'inc', 'flip', 'rotate', 'mask')
         defaults = (56, 4, 1, 0, 0, 0, 1)
-        udg_params = self.parse_image_params(text, index, defaults=defaults, path_id=path_id, names=param_names)
-        end, udg_path, crop_rect, addr, attr, scale, step, inc, flip, rotate, mask = udg_params
+        udg_params = self.parse_image_params(text, index, defaults=defaults, path_id=path_id, names=param_names, alt=True)
+        end, udg_path, alt, crop_rect, addr, attr, scale, step, inc, flip, rotate, mask = udg_params
         if end < len(text) and text[end] == ':':
-            mask_params = self.parse_image_params(text, end + 1, defaults=(step,), path_id=path_id, names=('addr', 'step'))
-            end, udg_path, crop_rect, mask_addr, mask_step = mask_params
+            mask_params = self.parse_image_params(text, end + 1, defaults=(step,), path_id=path_id, names=('addr', 'step'), alt=True)
+            end, udg_path, alt, crop_rect, mask_addr, mask_step = mask_params
         else:
             mask_params = None
             mask = 0
@@ -1317,12 +1322,15 @@ class HtmlWriter:
             udg.flip(flip)
             udg.rotate(rotate)
             self.write_image(udg_path, [[udg]], crop_rect, scale, mask)
-        return end, self.img_element(cwd, udg_path)
+        return end, self.img_element(cwd, udg_path, alt)
 
     def _expand_udgarray_with_frames(self, text, index, cwd):
         end, frame_params, fname = skoolmacro.parse_params(text, index, except_chars=' (')
         if not fname:
             raise MacroParsingError('Missing filename: #UDGARRAY{}'.format(text[index:end]))
+        alt = None
+        if '|' in fname:
+            fname, alt = fname.split('|', 1)
         img_path = self.image_path(fname, 'UDGImagePath')
         if self.need_image(img_path):
             frames = []
@@ -1344,7 +1352,7 @@ class HtmlWriter:
                 frame.delay = delay
                 frames.append(frame)
             self.write_animated_image(img_path, frames)
-        return end, self.img_element(cwd, img_path)
+        return end, self.img_element(cwd, img_path, alt)
 
     def expand_udgarray(self, text, index, cwd):
         # #UDGARRAYwidth[,attr,scale,step,inc,flip,rotate,mask];addr[,attr,step,inc][:addr[,step]];...[{x,y,width,height}](fname)
@@ -1355,22 +1363,22 @@ class HtmlWriter:
         udg_path_id = None
         param_names = ('width', 'attr', 'scale', 'step', 'inc', 'flip', 'rotate', 'mask')
         defaults = (56, 2, 1, 0, 0, 0, 1)
-        udg_array_params = self.parse_image_params(text, index, defaults=defaults, path_id=udg_path_id, names=param_names)
-        end, udg_path, crop_rect, width, attr, scale, step, inc, flip, rotate, mask = udg_array_params
+        udg_array_params = self.parse_image_params(text, index, defaults=defaults, path_id=udg_path_id, names=param_names, frame=True, alt=True)
+        end, udg_path, frame, alt, crop_rect, width, attr, scale, step, inc, flip, rotate, mask = udg_array_params
         udg_array = [[]]
         has_masks = False
         while end < len(text) and text[end] == ';':
             param_names = ('addr', 'attr', 'step', 'inc')
             defaults = (attr, step, inc)
-            udg_params = self.parse_image_params(text, end + 1, defaults=defaults, path_id=udg_path_id, chars='-x', ints=(1, 2, 3), names=param_names)
-            end, fname, crop_rect, udg_addr, udg_attr, udg_step, udg_inc = udg_params
+            udg_params = self.parse_image_params(text, end + 1, defaults=defaults, path_id=udg_path_id, chars='-x', ints=(1, 2, 3), names=param_names, frame=True, alt=True)
+            end, fname, frame, alt, crop_rect, udg_addr, udg_attr, udg_step, udg_inc = udg_params
             udg_addresses = self._get_udg_addresses(udg_addr, width)
             mask_addresses = []
             if end < len(text) and text[end] == ':':
                 param_names = ('addr', 'step')
                 defaults = (udg_step,)
-                mask_params = self.parse_image_params(text, end + 1, defaults=defaults, path_id=udg_path_id, chars='-x', ints=(1,), names=param_names)
-                end, fname, crop_rect, mask_addr, mask_step = mask_params
+                mask_params = self.parse_image_params(text, end + 1, defaults=defaults, path_id=udg_path_id, chars='-x', ints=(1,), names=param_names, frame=True, alt=True)
+                end, fname, frame, alt, crop_rect, mask_addr, mask_step = mask_params
                 if mask:
                     mask_addresses = self._get_udg_addresses(mask_addr, width)
             has_masks = has_masks or len(mask_addresses) > 0
@@ -1387,12 +1395,9 @@ class HtmlWriter:
         if not has_masks:
             mask = 0
 
-        if not fname:
-            raise MacroParsingError('Missing filename: #UDGARRAY{0}'.format(text[index:end]))
+        if not fname and frame is None:
+            raise MacroParsingError('Missing filename: #UDGARRAY{}'.format(text[index:end]))
 
-        fname, sep, frame = fname.partition('*')
-        if sep and not frame:
-            frame = fname
         if not fname and not frame:
             raise MacroParsingError('Missing filename or frame ID: #UDGARRAY{}'.format(text[index:end]))
 
@@ -1408,7 +1413,7 @@ class HtmlWriter:
             if need_image:
                 self.write_image(img_path, udg_array, crop_rect, scale, mask)
         if img_path:
-            return end, self.img_element(cwd, img_path)
+            return end, self.img_element(cwd, img_path, alt)
         return end, ''
 
     def expand_udgtable(self, text, index, cwd):
