@@ -150,7 +150,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
         self.write_text_file(skool, '{}.skool'.format(reffile[:-4]))
         output, error = self.run_skool2html('{} -d {} {} {} {}'.format(self._css_c(), self.odir, write_option, file_ids, reffile))
         self.assertEqual(error, '')
-        self.assertTrue(method_name in html_writer.call_dict, '{} was not called'.format(method_name))
+        self.assertIn(method_name, html_writer.call_dict, '{} was not called'.format(method_name))
         arg_list = html_writer.call_dict[method_name]
         exp_arg_list = exp_arg_list or [()]
         self.assertEqual(arg_list, exp_arg_list, '{}: {} != {}'.format(method_name, arg_list, exp_arg_list))
@@ -296,10 +296,8 @@ class Skool2HtmlTest(SkoolKitTestCase):
         game_dir = 'program'
         output, error = self.run_skool2html('{0} -d {1} -'.format(self._css_c(), self.odir))
         self.assertEqual(len(error), 0)
-        self.assertEqual(output[1], 'Using skool file: -')
-        self.assertEqual(output[2], 'Found no ref file for -')
-        self.assertEqual(output[3], 'Parsing standard input')
-        self.assertEqual(output[4], 'Creating directory {0}/{1}'.format(self.odir, game_dir))
+        self.assertEqual(output[1], 'Parsing skool file from standard input')
+        self.assertEqual(output[2], 'Creating directory {}/{}'.format(self.odir, game_dir))
         self.assertTrue(isfile(join(self.odir, game_dir, 'asm', '30000.html')))
 
     def test_output_dir_with_trailing_separator(self):
@@ -317,6 +315,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
         self.assertEqual(output[3], 'Creating directory {0}'.format(name))
 
     def test_html_writer_class(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         writer_module = '\n'.join((
             'import sys',
             'from skoolkit.skoolhtml import HtmlWriter',
@@ -333,9 +332,9 @@ class Skool2HtmlTest(SkoolKitTestCase):
         reffile = self.write_text_file(ref.format(module_dir, module_name), suffix='.ref')
         name = reffile[:-4]
         self.write_text_file('', '{0}.skool'.format(name))
-        output, error = self.run_skool2html('{0} -d {1} {2}'.format(self._css_c(), self.odir, reffile))
+        output, error = self.run_skool2html(reffile)
         self.assertEqual(error, '')
-        self.assertEqual(output[4], message)
+        self.assertEqual(output[3], message)
 
     def test_file_identification(self):
         # Test that a file named *.ref is treated as a ref file
@@ -343,7 +342,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
         self.write_text_file(path='{0}.skool'.format(reffile[:-4]))
         output, error = self.run_skool2html('{0} -d {1} {2}'.format(self._css_c(), self.odir, reffile))
         self.assertEqual(error, '')
-        self.assertTrue('write_index' in html_writer.call_dict)
+        self.assertIn('write_index', html_writer.call_dict)
 
         # Test that a file not named *.ref is treated as a skool file
         for suffix in ('.skool', '.sks', '.kit', ''):
@@ -374,7 +373,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
         self.assertEqual(error, '')
         html_writer = write_disassembly_args[0]
         self.assertEqual(len(html_writer.entries), 1)
-        self.assertTrue(30000 in html_writer.entries)
+        self.assertIn(30000, html_writer.entries)
 
     def test_image_writer_options(self):
         self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
@@ -541,6 +540,39 @@ class Skool2HtmlTest(SkoolKitTestCase):
         with self.assertRaisesRegexp(SkoolKitError, 'Cannot copy {0} to {1}: {1} is not a directory'.format(resource, dest_dir)):
             self.run_skool2html('{} -d {} {}'.format(self._css_c(), self.odir, reffile))
 
+    def test_option_S(self):
+        resource_dir = self.make_directory()
+        css_fname = 'foo.css'
+        cssfile = self.write_text_file(path='{}/{}'.format(resource_dir, css_fname))
+        reffile = self.write_text_file('[Game]\nStyleSheet={}'.format(css_fname), suffix='.ref')
+        self.write_text_file(path='{}.skool'.format(reffile[:-4]))
+        for option in ('-S', '--search'):
+            output, error = self.run_skool2html('{} {} -d {} {}'.format(option, resource_dir, self.odir, reffile))
+            self.assertEqual(error, '')
+            game_dir = os.path.join(self.odir, reffile[:-4])
+            self.assertTrue(os.path.isfile(os.path.join(game_dir, css_fname)))
+
+    def test_option_S_multiple(self):
+        css_fname = 'foo.css'
+        js_fname = 'bar.js'
+        ref = '\n'.join((
+            '[Game]',
+            'StyleSheet={}'.format(css_fname),
+            'JavaScript={}'.format(js_fname)
+        ))
+        reffile = self.write_text_file(ref, suffix='.ref')
+        resource_dir1 = self.make_directory()
+        resource_dir2 = self.make_directory()
+        resource_dir3 = self.make_directory()
+        cssfile = self.write_text_file(path='{}/{}'.format(resource_dir1, css_fname))
+        jsfile = self.write_text_file(path='{}/{}'.format(resource_dir2, js_fname))
+        self.write_text_file(path='{}/{}.skool'.format(resource_dir3, reffile[:-4]))
+        output, error = self.run_skool2html('-S {} --search {} -S {} -d {} {}'.format(resource_dir1, resource_dir2, resource_dir3, self.odir, reffile))
+        self.assertEqual(error, '')
+        game_dir = os.path.join(self.odir, reffile[:-4])
+        self.assertTrue(os.path.isfile(os.path.join(game_dir, css_fname)))
+        self.assertTrue(os.path.isfile(os.path.join(game_dir, js_fname)))
+
     def test_option_s(self):
         exp_preamble = [
             'skool2html.py searches the following directories for skool files, ref files,',
@@ -555,7 +587,8 @@ class Skool2HtmlTest(SkoolKitTestCase):
             os.path.join('.', 'resources'),
             os.path.join(os.path.expanduser('~'), '.skoolkit'),
             os.path.normpath('/usr/share/skoolkit'),
-            os.path.join(PACKAGE_DIR, 'resources')
+            os.path.join(PACKAGE_DIR, 'resources'),
+            'Any other directories specified by the -S/--search option'
         ]
         for option in ('-s', '--search-dirs'):
             output, error = self.run_skool2html(option, catch_exit=0)
@@ -578,8 +611,8 @@ class Skool2HtmlTest(SkoolKitTestCase):
         for option in ('-j', '--join-css'):
             output, error = self.run_skool2html('{} {} -d {} {}'.format(option, single_css, self.odir, reffile))
             self.assertEqual(error, '')
-            self.assertTrue(appending_msg.format(css1) in output)
-            self.assertTrue(appending_msg.format(css2) in output)
+            self.assertIn(appending_msg.format(css1), output)
+            self.assertIn(appending_msg.format(css2), output)
             self.assertTrue(os.path.isfile(single_css_f))
             self.assertFalse(os.path.isfile(os.path.join(game_dir, css1)))
             self.assertFalse(os.path.isfile(os.path.join(game_dir, css2)))
@@ -629,9 +662,9 @@ class Skool2HtmlTest(SkoolKitTestCase):
             for pages in ('Page1', 'Page1,Page2'):
                 output, error = self.run_skool2html('{} {} {}'.format(option, pages, reffile))
                 self.assertEqual(error, '')
-                self.assertEqual(write_disassembly_args[3], pages.split(','))
+                self.assertEqual(write_disassembly_args[4], pages.split(','))
         output, error = self.run_skool2html(reffile)
-        self.assertEqual(write_disassembly_args[3], ['Page1', 'Page2'])
+        self.assertEqual(write_disassembly_args[4], ['Page1', 'Page2'])
 
     def test_option_w_m(self):
         exp_arg_list = [
@@ -680,9 +713,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
     def test_option_V(self):
         for option in ('-V', '--version'):
             output, error = self.run_skool2html(option, err_lines=True, catch_exit=0)
-            self.assertEqual(len(output), 0)
-            self.assertEqual(len(error), 1)
-            self.assertEqual(error[0], 'SkoolKit {}'.format(VERSION))
+            self.assertEqual(['SkoolKit {}'.format(VERSION)], output + error)
 
     def test_option_p(self):
         for option in ('-p', '--package-dir'):
@@ -699,7 +730,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
         for option in ('-q', '--quiet'):
             output, error = self.run_skool2html('{0} {1} -d {2} -w i {3}'.format(self._css_c(), option, self.odir, reffile))
             self.assertEqual(error, '')
-            self.assertTrue(index_method in html_writer.call_dict, '{0} was not called'.format(index_method))
+            self.assertIn(index_method, html_writer.call_dict, '{0} was not called'.format(index_method))
             self.assertEqual(len(output), 0)
 
     def test_option_t(self):
@@ -710,9 +741,10 @@ class Skool2HtmlTest(SkoolKitTestCase):
             self.assertEqual(error, '')
             done = output[-1]
             search = re.search(pattern, done)
-            self.assertTrue(search is not None, '"{0}" is not of the form "{1}"'.format(done, pattern))
+            self.assertIsNot(search, None, '"{0}" is not of the form "{1}"'.format(done, pattern))
 
     def test_option_c(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         ref = TEST_WRITER_REF + '[Colours]\nRED=197,0,0'
         reffile = self.write_text_file(ref, suffix='.ref')
         self.write_text_file('', '{0}.skool'.format(reffile[:-4]))
@@ -723,12 +755,21 @@ class Skool2HtmlTest(SkoolKitTestCase):
                 ('Config', 'GameDir', 'test-c'),
                 ('ImageWriter', 'DefaultFormat', 'gif')
             ):
-                output, error = self.run_skool2html('{0} {1} {2}/{3}={4} -w i -d {5} {6}'.format(self._css_c(), option, section_name, param_name, value, self.odir, reffile))
+                output, error = self.run_skool2html('{} {}/{}={} {}'.format(option, section_name, param_name, value, reffile))
                 self.assertEqual(error, '')
                 section = html_writer.ref_parser.get_dictionary(section_name)
                 self.assertEqual(section[param_name], value, '{0}/{1}!={2}'.format(section_name, param_name, value))
             with self.assertRaisesRegexp(SkoolKitError, 'Malformed SectionName/Line spec: {0}'.format(sl_spec)):
-                self.run_skool2html('{0} {1} -w i -d {2} {3}'.format(option, sl_spec, self.odir, reffile))
+                self.run_skool2html('{} {} {}'.format(option, sl_spec, reffile))
+
+    def test_option_W(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
+        writer_class = '{}.TestHtmlWriter'.format(__name__)
+        skoolfile = self.write_text_file(suffix='.skool')
+        for option in ('-W', '--writer'):
+            output, error = self.run_skool2html('{} {} {}'.format(option, writer_class, skoolfile))
+            self.assertEqual(error, '')
+            self.assertEqual(html_writer.__class__, TestHtmlWriter)
 
     def test_option_T(self):
         reffile = self.write_text_file(TEST_WRITER_REF, suffix='.ref')
@@ -761,42 +802,47 @@ class Skool2HtmlTest(SkoolKitTestCase):
         self.assertEqual(actual_css, exp_css)
 
     def test_option_o(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         reffile = self.write_text_file(TEST_WRITER_REF, suffix='.ref')
         self.write_text_file('', '{0}.skool'.format(reffile[:-4]))
         for option in ('-o', '--rebuild-images'):
-            output, error = self.run_skool2html('{0} {1} -w "" -d {2} {3}'.format(self._css_c(), option, self.odir, reffile))
+            output, error = self.run_skool2html('{} {}'.format(option, reffile))
             self.assertEqual(error, '')
             self.assertTrue(html_writer.file_info.replace_images)
 
     def test_option_l(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         reffile = self.write_text_file(TEST_WRITER_REF, suffix='.ref')
         self.write_text_file('', '{0}.skool'.format(reffile[:-4]))
         for option in ('-l', '--lower'):
-            output, error = self.run_skool2html('{0} {1} -w "" -d {2} {3}'.format(self._css_c(), option, self.odir, reffile))
+            output, error = self.run_skool2html('{} {}'.format(option, reffile))
             self.assertEqual(error, '')
             self.assertEqual(html_writer.case, CASE_LOWER)
 
     def test_option_u(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         reffile = self.write_text_file(TEST_WRITER_REF, suffix='.ref')
         self.write_text_file('', '{0}.skool'.format(reffile[:-4]))
         for option in ('-u', '--upper'):
-            output, error = self.run_skool2html('{0} {1} -w "" -d {2} {3}'.format(self._css_c(), option, self.odir, reffile))
+            output, error = self.run_skool2html('{} {}'.format(option, reffile))
             self.assertEqual(error, '')
             self.assertEqual(html_writer.case, CASE_UPPER)
 
     def test_option_D(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         reffile = self.write_text_file(TEST_WRITER_REF, suffix='.ref')
         self.write_text_file('', '{0}.skool'.format(reffile[:-4]))
         for option in ('-D', '--decimal'):
-            output, error = self.run_skool2html('{0} {1} -w "" -d {2} {3}'.format(self._css_c(), option, self.odir, reffile))
+            output, error = self.run_skool2html('{} {}'.format(option, reffile))
             self.assertEqual(error, '')
             self.assertTrue(html_writer.parser.mode.decimal)
 
     def test_option_H(self):
+        self.mock(skool2html, 'write_disassembly', mock_write_disassembly)
         reffile = self.write_text_file(TEST_WRITER_REF, suffix='.ref')
         self.write_text_file('', '{0}.skool'.format(reffile[:-4]))
         for option in ('-H', '--hex'):
-            output, error = self.run_skool2html('{0} {1} -w "" -d {2} {3}'.format(self._css_c(), option, self.odir, reffile))
+            output, error = self.run_skool2html('{} {}'.format(option, reffile))
             self.assertEqual(error, '')
             self.assertTrue(html_writer.parser.mode.hexadecimal)
 
@@ -805,7 +851,7 @@ class Skool2HtmlTest(SkoolKitTestCase):
             output, error = self.run_skool2html(option, catch_exit=0)
             self.assertEqual(error, '')
             self.assertEqual(output[0], '[Colours]')
-            self.assertTrue('[Titles]' in output)
+            self.assertIn('[Titles]', output)
 
     def test_option_r(self):
         for option in ('-r', '--ref-sections'):
