@@ -377,7 +377,7 @@ class AsmWriterTest(SkoolKitTestCase):
 
     def _test_reference_macro(self, macro, def_link_text):
         writer = self._get_writer()
-        for link_text in ('',  '()', '(testing)', '(testing (nested) parentheses)'):
+        for link_text in ('', '()', '(testing)', '(testing (nested) parentheses)'):
             for anchor in ('', '#name', '#foo$bar'):
                 output = writer.expand('#{}{}{}'.format(macro, anchor, link_text))
                 self.assertEqual(output, link_text[1:-1] or def_link_text)
@@ -694,6 +694,11 @@ class AsmWriterTest(SkoolKitTestCase):
         self.assertEqual(output, '')
         self.assertEqual(writer.snapshot[addr], byte)
 
+    def test_macro_pops_empty_stack(self):
+        writer = self._get_writer()
+        prefix = ERROR_PREFIX.format('POPS')
+        self.assert_error(writer, '#POPS', 'Cannot pop snapshot when snapshot stack is empty', prefix)
+
     def test_macro_pushs(self):
         writer = self._get_writer()
         addr, byte = 32768, 64
@@ -1108,21 +1113,25 @@ class AsmWriterTest(SkoolKitTestCase):
             '; Routine at 32768',
             ';',
             '; Used by the routine at 32768.',
+            ';',
+            '; HL 32768',
             'c32768 LD A,B  ; This instruction is at 32768',
             '; This mid-routine comment is above 32769.',
             ' 32769 RET',
         ))
         self._get_asm(skool, warn=True)
         warnings = self.err.getvalue().split('\n')[:-1]
-        self.assertEqual(len(warnings), 8)
+        self.assertEqual(len(warnings), 10)
         self.assertEqual(warnings[0], 'WARNING: Comment contains address (32768) not converted to a label:')
         self.assertEqual(warnings[1], '; Routine at 32768')
         self.assertEqual(warnings[2], 'WARNING: Comment contains address (32768) not converted to a label:')
         self.assertEqual(warnings[3], '; Used by the routine at 32768.')
-        self.assertEqual(warnings[4], 'WARNING: Comment at 32768 contains address (32768) not converted to a label:')
-        self.assertEqual(warnings[5], '  LD A,B                  ; This instruction is at 32768')
-        self.assertEqual(warnings[6], 'WARNING: Comment above 32769 contains address (32769) not converted to a label:')
-        self.assertEqual(warnings[7], '; This mid-routine comment is above 32769.')
+        self.assertEqual(warnings[4], 'WARNING: Register description contains address (32768) not converted to a label:')
+        self.assertEqual(warnings[5], '; HL 32768')
+        self.assertEqual(warnings[6], 'WARNING: Comment at 32768 contains address (32768) not converted to a label:')
+        self.assertEqual(warnings[7], '  LD A,B                  ; This instruction is at 32768')
+        self.assertEqual(warnings[8], 'WARNING: Comment above 32769 contains address (32769) not converted to a label:')
+        self.assertEqual(warnings[9], '; This mid-routine comment is above 32769.')
 
     def test_warn_long_line(self):
         skool = '\n'.join((
@@ -1251,6 +1260,7 @@ class AsmWriterTest(SkoolKitTestCase):
             '; Input:a Some value',
             ';       b Some other value',
             '; Output:c The result',
+            ';        d',
             'c24605 RET',
         ))
         asm = self._get_asm(skool)
@@ -1263,6 +1273,33 @@ class AsmWriterTest(SkoolKitTestCase):
         self.assertEqual(asm[12], ';  Input:a Some value')
         self.assertEqual(asm[13], ';        b Some other value')
         self.assertEqual(asm[14], '; Output:c The result')
+        self.assertEqual(asm[15], ';        d')
+
+    def test_register_description_continuation_lines(self):
+        skool = '\n'.join((
+            '; @start',
+            '; Routine',
+            ';',
+            '; .',
+            ';',
+            '; BC This register description is long enough that it needs to be',
+            ';   .split over two lines',
+            '; DE Short register description',
+            '; HL Another register description that is long enough to need',
+            '; .  splitting over two lines',
+            'c40000 RET'
+        ))
+        asm = self._get_asm(skool)
+
+        exp_asm = [
+            '; BC This register description is long enough that it needs to be split over',
+            ';    two lines',
+            '; DE Short register description',
+            '; HL Another register description that is long enough to need splitting over',
+            ';    two lines',
+            '  RET'
+        ]
+        self.assertEqual(exp_asm, asm[2:2 + len(exp_asm)])
 
     def test_registers_upper(self):
         skool = '\n'.join((
@@ -1509,6 +1546,9 @@ class AsmWriterTest(SkoolKitTestCase):
             ';',
             '; @ignoreua',
             '; Description of routine at 32768.',
+            ';',
+            '; @ignoreua',
+            '; HL 32768',
             '; @ignoreua',
             'c32768 LD A,B ; This is the instruction at 32768',
             '; @ignoreua',
